str_description = """

    The data models/schemas for the PACS QR collection.

"""

from    pydantic            import BaseModel, Field
from    typing              import Optional, List, Dict
from    datetime            import datetime
from    enum                import Enum
from    pathlib             import Path
from    pfmongo.config      import settings

class stateResponse(BaseModel):
    database:str                    = ""
    collection:str                  = ""
    app:settings.App                = settings.appsettings
    mongo:settings.Mongo            = settings.mongosettings

class mongodbResponse(BaseModel):
    """The response model from the mongodb server"""
    status:bool                     = False
    message:str                     = ''
    response:dict                   = {}

# Connection status returns

class databaseConnectStatus(BaseModel):
    status:bool                     = False
    connected:bool                  = False
    existsAlready:bool              = False
    error:str                       = ""

class collectionConnectStatus(BaseModel):
    status:bool                     = False
    connected:bool                  = False
    existsAlready:bool              = False
    elements:int                    = 0
    error:str                       = ""

# API connection returns

class databaseDesc(BaseModel):
    status:bool                     = False
    otype:str                       = "database"
    host:str                        = ""
    port:int                        = -1
    name:str                        = ""
    info:databaseConnectStatus      = databaseConnectStatus()

class collectionDesc(BaseModel):
    status:bool                     = False
    database:databaseDesc           = databaseDesc()
    otype:str                       = "collection"
    databaseName:str                = ""
    name:str                        = ""
    fullName:str                    = ""
    info:collectionConnectStatus    = collectionConnectStatus()

# API usage returns

class showAllDBusage(BaseModel):
    """ response for getting a list of all databases in a mongoDB """
    status:bool                     = False
    otype:str                       = "accessing database names"
    databaseNames:list              = []
    info:databaseConnectStatus      = databaseConnectStatus()

class showAllcollectionsUsage(BaseModel):
    """ response for getting a list of all collections in a database """
    status:bool                     = False
    otype:str                       = "accessing collection names"
    collectionNames:list            = []
    info:databaseConnectStatus      = databaseConnectStatus()

class databaseNamesUsage(BaseModel):
    status:bool                     = False
    otype:str                       = "accessing database names"
    databaseNames:list              = []
    info:databaseConnectStatus      = databaseConnectStatus()

class collectionNamesUsage(BaseModel):
    status:bool                     = False
    otype:str                       = "accessing collection names"
    collectionNames:list            = []
    info:databaseConnectStatus      = databaseConnectStatus()

class DocumentAddUsage(BaseModel):
    """ response for adding a document to a collection """
    status:bool                     = False
    otype:str                       = "adding document"
    documentName:str                = ""
    document:dict                   = {}
    resp:dict                       = {}
    collection:collectionDesc       = collectionDesc()

class DocumentDeleteUsage(BaseModel):
    """ response for deleting a document from a collection """
    status:bool                     = False
    otype:str                       = "deleting document"
    documentName:str                = ""
    document:dict                   = {}
    resp:dict                       = {}
    collection:collectionDesc       = collectionDesc()

class CollectionDeleteUsage(BaseModel):
    """ response for deleting a collection from a database """
    status:bool                     = False
    otype:str                       = "deleting collection"
    collectionName:str              = ""
    resp:dict                       = {}
    collection:collectionDesc       = collectionDesc()

class dbDeleteUsage(BaseModel):
    """ response for deleting a database from a mongo server """
    status:bool                     = False
    otype:str                       = "deleting collection"
    dbName:str                      = ""
    resp:dict                       = {}
    db:databaseDesc                 = databaseDesc()

class DocumentListUsage(BaseModel):
    """ response for listing all documents from a collection """
    status:bool                     = False
    otype:str                       = "listing all documents"
    documentField:str               = ""
    documentList:list               = []
    resp:dict                       = {}
    collection:collectionDesc       = collectionDesc()

class DocumentSearchUsage(DocumentListUsage):
    """ response for listing all documents from a collection """
    otype:str                       = "searching all documents"

class DocumentGetUsage(BaseModel):
    """ response for getting adding a document from a collection """
    status:bool                     = False
    otype:str                       = "getting document"
    documentName:str                = ""
    document:dict                   = {}
    resp:dict                       = {}
    collection:collectionDesc       = collectionDesc()




import  os
from    pydantic_settings   import BaseSettings
from    pfmongo.models      import dataModel
from    typing              import Optional

class Mongo(BaseSettings):
    MD_URI:str              = "localhost:27017"
    MD_DB:str               = ""
    MD_COLLECTION:str       = ""
    MD_username:str         = "username"
    MD_password:str         = "password"
    MD_sessionUser:str      = ""
    stateDBname:str         = "db.txt"
    stateColname:str        = "collection.txt"
    stateDupname:str        = "duplicates.txt"
    stateHashes:str         = "hashes.txt"
    flattenSuffix:str       = "-flat"
    responseTruncDepth:int  = 4
    responseTruncSize:int   = 6000
    responseTruncOver:int   = 100000

class App(BaseSettings):
    logging:dataModel.loggingType = dataModel.loggingType.CONSOLE
    allowDuplicates:bool    = True
    noHashing:bool          = True
    donotFlatten:bool       = False
    beQuiet:bool            = False
    noResponseTruncSize:bool= True
    noComplain:bool         = False
    detailedOutput:bool     = False
    modelSizesPrint:bool    = False
    eventLoopDebug:bool     = False
    fontawesomeUse:bool     = True

logging_val:Optional[str]   = 'CONSOLE'
if 'LOGGING' in os.environ:
    logging_val             = os.environ['LOGGING']
logging_enum:dataModel.loggingType  = dataModel.loggingType.CONSOLE
if logging_val:
    try:
        logging_enum        = dataModel.loggingType[logging_val]
    except KeyError:
        logging_enum        = dataModel.loggingType.CONSOLE

mongosettings:Mongo         = Mongo()
appsettings:App             = App(logging = logging_enum)



import  click
from    argparse        import  Namespace
from    pfmisc          import  Colors as C
from    pfmongo         import  driver, env
from    pfmongo.models  import  responseModel
import  copy

NC = C.NO_COLOUR
GR = C.LIGHT_GREEN
CY = C.CYAN
PL = C.PURPLE

def options_add(options:Namespace) -> Namespace:
    localoptions:Namespace = copy.deepcopy(options)
    localoptions.do        = 'showAllDB'
    return localoptions

def showAll_asInt(options:Namespace) -> int:
    return driver.run_intReturn(options)

def showAll_asModel(options:Namespace) -> responseModel.mongodbResponse:
    return driver.run_modelReturn(options)

@click.command(cls = env.CustomCommand, help=f"""
list {PL}DATABASES{NC} containing data

SYNOPSIS
{CY}showall{NC}

DESC
This command shows all the populated databases available in a given mongodb
server. It accepts no arguments.

""")
@click.pass_context
def showAll(ctx:click.Context) -> int:
    # pudb.set_trace()
    return showAll_asInt(options_add(ctx.obj['options']))



from argparse import Namespace
import click
from pfmongo import driver, env
from argparse import Namespace
from pfmisc import Colors as C
from pfmongo.models import responseModel, fsModel
from pathlib import Path
import pudb

from pfmongo.commands.document import showAll as doc
from pfmongo.commands.state import showAll as state
from pfmongo.commands.database import showAll as mdb
from pfmongo.commands.database import connect as dbconnect
from pfmongo.commands.collection import showAll as col
from pfmongo.commands.collection import connect as colconnect
import pfmongo.commands.smash as smash
import copy
from types import SimpleNamespace

NC = C.NO_COLOUR
GR = C.GREEN
CY = C.CYAN
YL = C.YELLOW


def options_add(path: str, options: Namespace, create: bool = False) -> Namespace:
    localoptions: Namespace = copy.deepcopy(options)
    localoptions.beQuiet = True
    localoptions.do = "cd"
    localoptions.cd = SimpleNamespace(**{"path": Path(path), "create": create})
    return localoptions


def path_to_dbCol(options: Namespace) -> tuple:
    path: Path = options.cd.path
    db: str = ""
    collection: str = ""
    match len(path.parts):
        case _ if len(path.parts) == 2:
            (root, db) = path.parts
        case _ if len(path.parts) > 2:
            (root, db, collection) = path.parts[:3]
    return (db, collection)


def toParent(options: Namespace) -> fsModel.cdResponse:
    return changeDirectory(options_add(options.cd.path.parent, options))


def toDir(options: Namespace) -> fsModel.cdResponse:
    (db, collection) = path_to_dbCol(options)
    return changeDirectory(options_add(f"/{db}/{collection}", options))


def fullPath_resolve(options: Namespace) -> Namespace:
    def navigate_cd():
        ups: str = "../"
        path: Path = smash.cwd(options)
        cd = str(options.cd.path)
        if cd.startswith(ups):
            if not cd.endswith("/"):
                cd += "/"
            levels_up: int = cd.count(ups)
            try:
                path = path.parents[levels_up - 1] if levels_up > 1 else path.parent
            except:
                path = Path("/")
            cd = cd[len(ups) * levels_up :]
        options.cd.path = path / cd

    def filter_toFirstStarInPath(path: Path) -> Path:
        index = next(
            (i for i, part in enumerate(path.parts) if "*" in part), len(path.parts)
        )
        return Path(*path.parts[:index])

    options.cd.path = filter_toFirstStarInPath(options.cd.path)
    if options.cd.path.is_absolute():
        return options
    if options.cd.path == Path("."):
        options.cd.path = smash.cwd(options)
    elif options.cd.path == Path(".."):
        options.cd.path = smash.cwd(options).parent
    elif smash.cwd(options) == Path("/") and not str(options.cd.path).startswith(".."):
        options.cd.path = Path("/") / options.cd.path
    else:
        navigate_cd()
    return options


def db_isListed(db: str, options: Namespace) -> fsModel.cdResponse:
    fsPath: fsModel.cdResponse = fsModel.cdResponse()
    if (
        db
        not in mdb.showAll_asModel(
            driver.settmp(options, [{"do": "showAllDB"}])
        ).message
    ):
        fsPath.error = f"database '{db}' not in mongo"
        fsPath.status = False
        fsPath.code = 2
        fsPath.state["database"] = "void"
    else:
        fsPath.state["database"] = "exists"
    return fsPath


def db_connect(db: str, options: Namespace) -> fsModel.cdResponse:
    fsPath: fsModel.cdResponse = fsModel.cdResponse()
    fsPath.path = options.cd.path
    if dbConnectFail := dbconnect.connectTo_asInt(dbconnect.options_add(db, options)):
        fsPath.error = f"could not connect to database '{db}'"
        fsPath.status = False
        fsPath.code = 3
        return fsPath
    return fsPath


def col_isListed(db: str, collection: str, options: Namespace) -> fsModel.cdResponse:
    fsPath: fsModel.cdResponse = fsModel.cdResponse()
    if (
        collection
        not in col.showAll_asModel(
            driver.settmp(options, [{"do": "showAllCollections"}])
        ).message
    ):
        fsPath.error = f"collection '{collection}' not in database '{db}'"
        fsPath.status = False
        fsPath.code = 4
        fsPath.state["collection"] = "void"
    else:
        fsPath.state["collection"] = "exists"
    return fsPath


def col_connect(db: str, collection: str, options: Namespace) -> fsModel.cdResponse:
    fsPath: fsModel.cdResponse = fsModel.cdResponse()
    fsPath.path = options.cd.path
    if colCollectFail := colconnect.connectTo_asInt(
        colconnect.options_add(collection, options)
    ):
        fsPath.error = (
            f"could not connect to collection '{collection}' in databse '{db}'"
        )
        fsPath.status = False
        fsPath.code = 5
    return fsPath


def is_root(options: Namespace) -> bool:
    b_ret: bool = False
    if not len(options.cd.path.parents):
        env.DBname_stateFileSave(options, "")
        env.collectionName_stateFileSave(options, "")
        b_ret = True
    return b_ret


def is_path_too_long(options) -> fsModel.cdResponse:
    fsPath: fsModel.cdResponse = fsModel.cdResponse()
    if len(options.cd.path.parents) > 2:
        fsPath.error = f"{options.cd.path}: path too long"
        fsPath.status = False
        fsPath.code = 1
    return fsPath


def path_connect(options: Namespace) -> fsModel.cdResponse:
    fsPath: fsModel.cdResponse = fsModel.cdResponse()
    connect: fsModel.cdResponse = fsModel.cdResponse()
    connect.path = options.cd.path
    if (fsPath := is_path_too_long(options)).code:
        return fsPath
    if is_root(options):
        return connect
    (db, collection) = path_to_dbCol(options)
    if db and not (fsPath := db_isListed(db, options)).status:
        if not options.cd.create:
            return fsPath
    connect.state["database"] = fsPath.state["database"]
    if db and not (fsPath := db_connect(db, options)).status:
        return fsPath
    if collection and not (fsPath := col_isListed(db, collection, options)).status:
        if not options.cd.create:
            return fsPath
    connect.state["collection"] = fsPath.state["collection"]
    if collection and not (fsPath := col_connect(db, collection, options)).status:
        return fsPath
    connect.status = True
    connect.message = f"successfully connected path to {connect.path}"
    connect.code = 0
    env.DBname_stateFileSave(options, db)
    env.collectionName_stateFileSave(options, collection)
    return connect


def path_lengthOK(options: Namespace) -> bool:
    return True if len(options.cd.path.parents.parts == 2) else False


def changeDirectory(options: Namespace) -> fsModel.cdResponse:
    options = fullPath_resolve(options)
    cdResponse: fsModel.cdResponse = fsModel.cdResponse()
    if (cdResponse := path_connect(options)).code:
        print(cdResponse.error)
    return cdResponse


@click.command(
    cls=env.CustomCommand,
    help=f"""
change {YL}directory{NC}

SYNOPSIS
{CY}cd {YL}[--create] <path>{NC}

DESC
The {YL}cd{NC} command "changes directory" within a mongodb between a
{CY}<database>{NC} level at the root `/` and a {CY}<collection>{NC} within that
{CY}<database>{NC}. Since there are only ever two "directories" in a
mongodb, trees are not especially nested.

Note that the underlying behaviour is subtly different to a standard
filesystem, and this 'cd' can also "create" a path/directory with if called
with {YL}--create{NC}.

""",
)
@click.argument("path", required=True)
@click.option(
    "--create", is_flag=True, help="If set, also create this path if it does not exist."
)
@click.pass_context
def cd(ctx: click.Context, path: str, create: bool) -> int:
    # pudb.set_trace()
    return changeDirectory(options_add(path, ctx.obj["options"], create)).code



import click
import click
import pudb
from typing import Any
import readline
import sys
from tabulate import tabulate
from pfmongo import pfmongo, __main__
from pfmongo import driver, env
from argparse import Namespace
from pfmisc import Colors as C
from pfmongo.models import responseModel
from typing import cast
from argparse import Namespace
from pfmongo.commands.state import showAll as state
from click.testing import CliRunner
from pathlib import Path
import ast
from fortune import fortune
import copy
import pfmongo.commands.fop.prompt as prompt
from pfmongo.config import settings
import subprocess
from typing import Optional, Callable, Union

# from    ansi2html               import Ansi2HTMLConverter
from pfmongo.commands.slib import tabc

NC = C.NO_COLOUR
GR = C.GREEN
CY = C.CYAN
YL = C.YELLOW

fscommand: list = [
    "sg",
    "ls",
    "cat",
    "rm",
    "cd",
    "mkcd",
    "imp",
    "exp",
    "prompt",
    "pwd",
    "quit",
    "exit",
    "fortune",
    "banner",
]

fscommand_noArgs: list = ["prompt", "pwd", "quit", "exit"]


def pipe_split(command: str) -> list:
    parts: list[str] = command.split("|", 1)
    return parts


def smash_output(command: str) -> str:
    output: str = meta_parse(command)
    if not output:
        ret = CliRunner().invoke(__main__.app, command.split(), color=True)
        output = ret.output
    return output


def smash_execute(
    command: str,
    f: Optional[Callable[[str, list[str]], subprocess.CompletedProcess]] = None,
) -> Union[bytes, str]:
    cmdpart: list = pipe_split(command)
    smash_ret: str = smash_output(cmdpart[0])
    result: str = smash_ret
    if len(cmdpart) > 1 and f:
        process: subprocess.CompletedProcess = f(smash_ret, cmdpart)
        result = f"exec: '{process.args}', returncode: {process.returncode}"
        result = ""
    return result


def pipe_handler(previous_input: str, cmdpart: list) -> subprocess.CompletedProcess:
    cmds = [c.strip() for c in cmdpart]
    shell_command = "|".join(cmds[1:])
    result: subprocess.CompletedProcess = subprocess.run(
        shell_command, input=previous_input, shell=True, capture_output=False, text=True
    )
    # converter:Ansi2HTMLConverter    = Ansi2HTMLConverter()
    # output:str                      = converter.convert(result.stdout)
    return result


def command_parse(command: str) -> str:
    fscall: list = [s for s in fscommand if command.lower().startswith(s)]
    if fscall:
        command = f"fs {command}"
    if command == "help":
        command = "fs --help"
    return command


def state_getModel(options: Namespace) -> responseModel.mongodbResponse:
    return state.showAll_asModel(driver.settmp(options, [{"beQuiet": True}]))


def cwd(options: Namespace) -> Path:
    model: responseModel.mongodbResponse = state_getModel(options)
    if model.message == "/":
        return Path("/")
    else:
        return Path("/" + model.message)


def prompt_get(options: Namespace) -> str:
    pathColor: str = prompt.prompt_do(prompt.options_add(options)).message
    return f"{CY}({settings.mongosettings.MD_sessionUser}@smash){NC}{pathColor}$>"


def command_get(options: Namespace) -> str:
    userInput: str = tabc.userInput_get(options)
    fscmd: str = f"{userInput}".strip()
    # pudb.set_trace()
    return fscmd


def meta_parse(command: str) -> str:
    output: str = ""
    if "quit" in command.lower() or "exit" in command.lower():
        sys.exit(0)
    if command == "banner":
        output = introbanner_generate()
    if command == "fortune":
        output = env.tabulate_message(fortune(), f"{YL}fortune{NC}")
    return output


def introbanner_generate() -> str:
    title: str = (
        f"{CY}s{YL}imple pf{CY}m{YL}ongo{NC} {CY}a{YL}pplication {CY}sh{YL}ell{NC}"
    )
    banner: str = f"""
{CY}░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░{NC}
{CY}░███████╗███╗░░░███╗░█████╗░███████╗██╗░░██╗██╗░{NC}
{CY}░██╔════╝████╗░████║██╔══██╗██╔════╝██║░░██║██║░{NC}
{CY}░███████╗██╔████╔██║███████║███████╗███████║██║░{NC}
{CY}░╚════██║██║╚██╔╝██║██╔══██║╚════██║██╔══██║╚═╝░{NC}
{CY}░███████║██║░╚═╝░██║██║░░██║███████║██║░░██║██╗░{NC}
{CY}░╚══════╝╚═╝░░░░░╚═╝╚═╝░░╚═╝╚══════╝╚═╝░░╚═╝╚═╝░{NC}
{CY}░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░░{NC}
"""
    intro: str = f"""
Welcome to {CY}smash{NC}, a simple shell to {YL}pfmongo{NC} that
allows you to directly call various subcommands.

If you really want to {CY}smash{NC} things up, the shell allows
you to call the {YL}fs{NC} subcommands without the {YL}fs{NC}, giving
the impression of being in a more standard un*x land.

Some useful commands:
 ▙▄ {YL}--help{NC} for a list of {YL}all{NC} commands.
 ▙▄ {YL}banner{NC} to see this amazing banner again.
 ▙▄ {YL}fortune{NC} since every shell needs this.
 ▙▄ {YL}help{NC} for a list of {YL}fs{NC} commands.
 ▙▄ {YL}quit{NC} or {YL}exit{NC} to return to the system.

Enjoy your stay and please remain on the trails!
Oh, and don't feed the wildlife. It's not good
for them.

Have a {CY}smash{NC}ing good time!
"""
    return env.tabulate_message(banner + intro, title)


@click.command(
    cls=env.CustomCommand,
    help="""
shell interface for running commands

An extremely "simple" pfmongo {YL}shell{NC}. Run commands from a shell-esque
interface that harkens back to the days of /bin/ash!

""",
)
@click.option("--prompt", is_flag=True, help="If set, print the CFS cwd as prompt")
@click.pass_context
def smash(ctx: click.Context, prompt) -> None:
    print(introbanner_generate())
    options: Namespace = ctx.obj["options"]
    while True:
        print(smash_execute(command_parse(command_get(options)), pipe_handler))



from    pfmisc      import  Colors  as C
import  click
import  pudb
from    pfmongo                 import env
from    pfmongo.commands.dbop   import connect, showAll, deleteDB


NC  = C.NO_COLOUR
GR  = C.GREEN
CY  = C.CYAN

@click.group(cls = env.CustomGroup, help=f"""
commands suitable for database focus: {GR}show, connect, deletedb{NC}

This command group provides mongo "database" level commands, allowing
connection to a new database, deletion of an existing database, and
listing all existing databases.

""")
def database():
    pass

database.add_command(connect.connect)
database.add_command(showAll.showAll)
database.add_command(deleteDB.deleteDB)



import  click
from    pfmongo         import  driver, env
from    argparse        import  Namespace
from    pfmisc          import  Colors as C
from    pfmongo.models  import  responseModel, fsModel
from    pathlib         import  Path
import  copy
import  ast
import  pudb

from    pfmongo.commands.document   import showAll as doc
from    pfmongo.commands.state      import showAll as state
import  pfmongo.commands.smash      as smash

NC  = C.NO_COLOUR
GR  = C.GREEN
CY  = C.CYAN
PL  = C.PURPLE
YL  = C.YELLOW

def options_add(options:Namespace) -> Namespace:
    localoptions:Namespace  = copy.deepcopy(options)
    localoptions.beQuiet    = True
    return localoptions

def pwd_level(options:Namespace) -> str:
    match len(smash.cwd(options).parts):
        case 1: return "root"
        case 2: return "database"
        case 3: return "collection"
        case _: return "unknown"

def dir_level(dir:fsModel.cdResponse) -> str:
    match len(dir.path.parts):
        case 1: return "root"
        case 2: return "database"
        case 3: return "collection"
        case _: return "unknown"

def pwd_do(options:Namespace) -> fsModel.cdResponse:
    pathResp            = fsModel.cdResponse()
    resp:responseModel.mongodbResponse   = state.showAll_asModel(options)
    if resp.message     == '/':
        pathResp.path   = Path(resp.message)
    else:
        pathResp.path   = Path('/') /  Path(resp.message)
    pathResp.status     = True
    return pathResp

def pwd_asInt(options):
    pathResp:fsModel.cdResponse = pwd_do(options)
    print(pathResp.path)
    return pathResp.code

@click.command(cls = env.CustomCommand, help=f"""
print working {YL}directory{NC}

SYNOPSIS
{CY}pwd{NC}

DESC
This command simply returns the current "working directory".

""")
@click.pass_context
def pwd(ctx:click.Context) -> int:
    # pudb.set_trace()
    return pwd_asInt(options_add(ctx.obj['options']))



import  click
from    argparse        import  Namespace
from    pfmongo         import  env, driver
from    pfmisc          import  Colors as C

from    pfmongo.commands.dbop   import  connect as db
from    pfmongo.models          import  responseModel
import  pudb
import  copy

NC  = C.NO_COLOUR
GR  = C.GREEN
CY  = C.CYAN
PL  = C.PURPLE
YL  = C.YELLOW

from pfmongo.models.dataModel import messageType

def options_add(database:str, options:Namespace) -> Namespace:
    localoptions:Namespace  = copy.deepcopy(options)
    localoptions.do         = 'deleteDB'
    localoptions.argument   = database
    return localoptions

def DB_connectToTarget(options:Namespace) -> str:
    currentDB:str  = env.DBname_get(options)
    if currentDB != options.argument:
        options.do      = 'connectDB'
        db.connectTo_asInt(options)
    return options.argument

def DBdel_setupFail(options: Namespace) -> int:
    if env.env_failCheck(options):
        return 100
    DB_connectToTarget(options)
    options.do          = 'deleteDB'
    return 0

def DBdel_asInt(options:Namespace) -> int:
    fail:int            = 0
    if (fail := DBdel_setupFail(options)):
        return fail
    return driver.run_intReturn(options)

def DBdel_asModel(options:Namespace) -> responseModel.mongodbResponse:
    model:responseModel.mongodbResponse = responseModel.mongodbResponse()
    if DBdel_setupFail(options):
        model.message   = 'env failure'
        return model
    return driver.run_modelReturn(options)

@click.command(cls = env.CustomCommand, help=f"""
delete an entire {PL}DATABASE{NC}

SYNOPSIS
{CY}deletedb {YL}<DATABASE>{NC}

DESC
This subcommand removes an entire {YL}DATABASE{NC} immediately.
Use with care! No confirmation is asked by the system!

""")
@click.argument('database',
                required = True)
@click.pass_context
def deleteDB(ctx:click.Context, database:str) -> int:
    # pudb.set_trace()
    return DBdel_asInt(options_add(database, ctx.obj['options']))



import  click
import  pudb
from    pfmongo                 import  driver, env
import  pfmongo.pfmongo
from    argparse                import  Namespace
import  json
from    pfmisc                  import  Colors as C
from    pfmongo.config          import  settings
from    pfmongo.models          import  responseModel
from    typing                  import  Tuple, cast, Callable
from    pfmongo.commands.docop  import  add
from    pathlib                 import  Path
import  copy

NC  = C.NO_COLOUR
GR  = C.GREEN
CY  = C.CYAN
PL  = C.PURPLE
YL  = C.YELLOW

def filename_get(path:str) -> str:
    filename:str = Path(path).name
    return filename

def options_add(file:str, options:Namespace) -> Namespace:
    localoptions:Namespace  = copy.deepcopy(copy.deepcopy(options))
    localoptions.beQuiet    = True
    localoptions.do         = 'import'
    localoptions.argument   = {"file": file}
    return localoptions

def imp_do(options:Namespace) -> responseModel.mongodbResponse:
    imp:responseModel.mongodbResponse   = add.documentAdd_asModel(
                                            add.options_add(
                                                options.argument['file'],
                                                filename_get(options.argument['file']),
                                                options
                                            )
                                        )
    return imp

def imp_asInt(options:Namespace) -> int:
    imp:responseModel.mongodbResponse   = imp_do(options)
    print(imp.message)
    return 0

@click.command(cls = env.CustomCommand, help=f"""
import a {CY}file{NC} as a {YL}document{NC}

SYNOPSIS
{CY}imp {YL}<file>{NC}

DESC
Import a {YL}file{NC} from the host system into the mongodb as a {YL}document{NC}.

The {YL}file{NC} can have a "path" prefix, denoting a database and collection to
use for the import.

""")
@click.argument('file',
                required = True)
@click.pass_context
def imp(ctx:click.Context, file:str) -> int:
    pudb.set_trace()
    return imp_asInt(options_add(file, ctx.obj['options']))





from    argparse        import Namespace
import  click
from    pfmisc          import Colors as C
from    pfmongo         import driver, env
from    pfmongo.models  import responseModel
from    pfmongo.config  import settings
from    typing          import Tuple
import  copy
NC = C.NO_COLOUR
GR = C.LIGHT_GREEN
CY = C.CYAN
PL = C.PURPLE
YL = C.YELLOW


def options_add(collection:str, options:Namespace) -> Namespace:
    localoptions            = copy.deepcopy(options)
    localoptions.do         = 'connectCollection'
    localoptions.argument   = collection
    return localoptions

def is_shadowCollection(collection:str) -> Tuple[str, bool]:
    isShadow:bool = True if collection.endswith(settings.mongosettings.flattenSuffix) \
                    else False
    return collection, isShadow

def baseCollection_getAndConnect(options:Namespace) -> Namespace:
    localoptions        = copy.deepcopy(options)
    localoptions.collectionName = env.collectionName_get(localoptions)
    currentCol:str      = env.collectionName_get(options)
    if currentCol.endswith(settings.mongosettings.flattenSuffix):
        currentCol      = currentCol.rstrip(settings.mongosettings.flattenSuffix)
        localoptions    = collection_connect(currentCol, options)
    return localoptions

def shadowCollection_getAndConnect(options:Namespace) -> Namespace:
    localoptions        = copy.deepcopy(options)
    localoptions.collectionName = env.collectionName_get(localoptions)
    shadowCol:str       = ""
    if not (currentCol := is_shadowCollection(env.collectionName_get(options)))[1]:
        shadowSuffix:str    = settings.mongosettings.flattenSuffix
        shadowCol:str       = currentCol[0] + shadowSuffix
        localoptions        = collection_connect(shadowCol, options)
    return localoptions

def collection_connect(collection:str, options:Namespace) -> Namespace:
    """ Returns a copy of the options with the following
        state changes:

            1. 'runRet' contains the int result of doing a connection
                        to the <collection>. This run is also performed
                        on its own options copy.
            2. 'collectionName' in the original copy is updated to the
                                <collection>.

        Note the original options passed into this method is *unchanged*!
    """
    return driver.settmp(options,
            [{'runRet':
                driver.run_intReturn(
                    driver.settmp(options,
                        [{'do': 'connectCollection'}, {'argument': collection}])
                    )
            },
            {'collectionName': collection}]
    )

def connectTo_asInt(options:Namespace) -> int:
    return driver.run_intReturn(options)

def connectTo_asModel(options:Namespace) -> responseModel.mongodbResponse:
    return driver.run_modelReturn(options)

@click.command(cls=env.CustomCommand, help=f"""
associate context with {PL}COLLECTION{NC}

SYNOPSIS
{CY}connect {YL}<COLLECTION>{NC}

DESC
This command connects to a mongo collection called {YL}COLLECTION{NC}
within a mongo database.

A mongodb "server" can contain several "databases", each of which
contains several "collections".

A {YL}COLLECTION{NC} is the second level of organization in a monogdb.

""")
@click.argument('collection',
                required = True)
@click.pass_context
def connect(ctx:click.Context, collection:str) -> int:
    # pudb.set_trace()
    return connectTo_asInt(options_add(collection, ctx.obj['options']))



import  click
from    argparse        import  Namespace
from    pfmongo         import  env, driver
from    pfmisc          import  Colors as C

from    pfmongo.commands.clop       import connect as collection
from    pfmongo.models              import responseModel
import  pudb
import  copy

NC  = C.NO_COLOUR
GR  = C.GREEN
CY  = C.CYAN
PL  = C.PURPLE
YL  = C.YELLOW

from pfmongo.models.dataModel import messageType


def options_add(collection:str, options:Namespace) -> Namespace:
    localoptions:Namespace  = copy.deepcopy(options)
    localoptions.do         = 'deleteCollection'
    localoptions.argument   = collection
    return localoptions

def col_connectToTarget(options:Namespace) -> str:
    currentCol:str      = env.collectionName_get(options)
    if currentCol != options.argument:
        options.do      = 'connectCollection'
        collection.connectTo_asInt(options)
    return options.argument

def collectiondel_setup(options: Namespace) -> int:
    if env.env_failCheck(options):
        return 100
    col_connectToTarget(options)
    options.do          = 'deleteCollection'
    return 0

def collectiondel_asInt(options:Namespace) -> int:
    fail:int            = 0
    if (fail := collectiondel_setup(options)):
        return fail
    return driver.run_intReturn(options)

def collectiondel_asModel(options:Namespace) -> responseModel.mongodbResponse:
    model:responseModel.mongodbResponse = responseModel.mongodbResponse()
    fail:int            = 0
    if (fail := collectiondel_setup(options)):
        model.message   = 'env failure'
        return model
    return driver.run_modelReturn(options)

@click.command(cls=env.CustomCommand, help=f"""
delete an entire {PL}COLLECTION{NC}

SYNOPSIS
{CY}deletecol {YL}<COLLECTION>{NC}

DESC
This subcommand removes an entire {YL}COLLECTION{NC} immediately.
Use with care! The system does not ask for confirmation!
""")
@click.argument('collection',
                required = True)
@click.pass_context
def deleteCol(ctx:click.Context, collection:str) -> int:
    # pudb.set_trace()
    # options:Namespace   = ctx.obj['options']
    return collectiondel_asInt(options_add(collection, ctx.obj['options']))



import click
from pfmongo import driver, env
from argparse import Namespace
from pfmisc import Colors as C
from pfmongo.models import fsModel, responseModel
from pathlib import Path
import ast
import pudb
import copy
import re
import fnmatch
import pfmongo.commands.smash as smash
import pfmongo.commands.fop.cd as cd
from pfmongo.commands.document import showAll as doc
from pfmongo.commands.database import showAll as db
from pfmongo.commands.collection import showAll as col
from pfmongo.commands.fop.pwd import pwd_level

from types import SimpleNamespace
import fontawesome as fa

NC = C.NO_COLOUR
GR = C.GREEN
CY = C.CYAN
PL = C.PURPLE
YL = C.YELLOW


def options_add(path: str, attribs: str, long: bool, options: Namespace) -> Namespace:
    localoptions = copy.deepcopy(options)
    localoptions.do = "ls"
    localoptions.beQuiet = True
    if not path:
        path = "."
    localoptions.argument = SimpleNamespace(
        **{"path": Path(path), "attribs": attribs, "long": long}
    )
    return localoptions


def objectSymbol_resolve(resp: responseModel.mongodbResponse) -> str:
    symbol: str = ""
    match type(resp.response["connect"]):
        case responseModel.showAllDBusage:
            symbol = fa.icons["database"]
        case responseModel.showAllcollectionsUsage:
            symbol = fa.icons["table"]
        case responseModel.DocumentListUsage:
            symbol = fa.icons["file"]
        case _:
            symbol = fa.icons["file"]
    return symbol + " "


def resp_process(resp: responseModel.mongodbResponse) -> str:
    rstr: str = ""
    file: list = ast.literal_eval(resp.message)
    # pudb.set_trace()
    for f in file:
        rstr += objectSymbol_resolve(resp) + f + "  "
    return rstr


def ls_db(options: Namespace) -> responseModel.mongodbResponse:
    resp = db.showAll_asModel(
        driver.settmp(db.options_add(options), [{"beQuiet": True}])
    )
    return resp


def ls_collection(options: Namespace) -> responseModel.mongodbResponse:
    resp = col.showAll_asModel(
        driver.settmp(col.options_add(options), [{"beQuiet": True}])
    )
    return resp


def file_filter(
    resp: responseModel.mongodbResponse, options: Namespace
) -> responseModel.mongodbResponse:
    if options.argument.path != Path("."):
        files: list[str] = eval(resp.message)
        filtered: list[str] = [
            e for e in files if fnmatch.fnmatch(e, options.argument.path)
        ]
        resp.message = "[" + ", ".join([f"'{e}'" for e in filtered]) + "]"
    return resp


def ls_doc(options: Namespace) -> responseModel.mongodbResponse:
    resp = doc.showAll_asModel(
        driver.settmp(doc.options_add("_id", options), [{"beQuiet": True}])
    )
    if resp.message:
        resp.message = ls_msgParse(resp.message)
    return resp


def ls_msgParse(lstr: str) -> str:
    return re.sub(r"ObjectId\('([\w]+)'\)", r"'\1'", lstr)


def ls_fargsUpdate(options: Namespace) -> Namespace:
    localoptions: Namespace = copy.deepcopy(options)
    cwdSet: set[str] = set(smash.cwd(options).parts[-1:])
    argSet: set[str] = set(options.argument.path.parts[-1:])
    try:
        localoptions.argument.path = (argSet - cwdSet).pop()
    except KeyError:
        localoptions.argument.path = Path(".")
    return localoptions


def ls_do(options: Namespace) -> tuple[int, responseModel.mongodbResponse]:
    cwd: Path = smash.cwd(options)
    resp: responseModel.mongodbResponse = responseModel.mongodbResponse()
    cdResp: fsModel.cdResponse = fsModel.cdResponse()
    # pudb.set_trace()
    # cd.changeDirectory(cd.options_add(options.argument.path, options))
    if not (
        cdResp := cd.toDir(
            cd.fullPath_resolve(cd.options_add(options.argument.path, options))
        )
    ).status:
        resp.message = cdResp.message + " " + cdResp.error
        return 1, resp

    loptions = ls_fargsUpdate(options)
    match pwd_level(loptions):
        case "root":
            resp = ls_db(loptions)
        case "database":
            resp = ls_collection(loptions)
        case "collection":
            resp = ls_doc(loptions)
        case "_":
            resp = ls_db(loptions)
    file_filter(resp, loptions)
    if not options.beQuiet:
        resp_process(resp)
    ret: int = 0
    if not resp.message:
        ret = 1
    cd.changeDirectory(cd.options_add(str(cwd), options))
    return ret, resp


def ls_asInt(options: Namespace) -> int:
    ret, resp = ls_do(options)
    return ret


def ls_asModel(options: Namespace) -> responseModel.mongodbResponse:
    ret, resp = ls_do(options)
    return resp


@click.command(
    cls=env.CustomCommand,
    help=f"""
list {YL}path{NC}

SYNOPSIS
{CY}ls {YL}[--long] [--human] [<path>]{NC}

ARGS
This command lists the objects (files and directories) that are at a given
path. This path can be a directory, in which case possibly multiple objects
are listed, or it can be a single file in which case information about that
single file is listed.

The {YL}--long{NC} flag triggers a detailed listing, showing analogues to
the document or "file" {CY}owner{NC}, {CY}size{NC}, and creation {CY}date{NC}.
This assumes that the document entry has these fields encoded, which is only
true for files uploaded using {YL}pfmongo{NC}.


""",
)
@click.argument("path", required=False)
@click.option(
    "--attribs",
    required=False,
    help="A comma separated list of file attributes to return/print",
)
@click.option("--long", is_flag=True, help="If set, use a long listing format")
@click.pass_context
def ls(ctx: click.Context, path: str, attribs: str, long: bool) -> int:
    # pudb.set_trace()
    ret, resp = ls_do(options_add(path, attribs, long, ctx.obj["options"]))
    print(resp_process(resp))
    return ret



import  sys
import  re
from    setuptools          import setup

_version_re = re.compile(r"(?<=^__version__ = (\"|'))(.+)(?=\"|')")

def get_version(rel_path: str) -> str:
    """
    Searches for the ``__version__ = `` line in a source code file.

    https://packaging.python.org/en/latest/guides/single-sourcing-package-version/
    """
    with open(rel_path, 'r') as f:
        matches = map(_version_re.search, f)
        filtered = filter(lambda m: m is not None, matches)
        version = next(filtered, None)
        if version is None:
            raise RuntimeError(f'Could not find __version__ in {rel_path}')
        return version.group(0)

requirements    = []
with open('requirements.txt') as f:
    requirements = [line.strip() for line in f.readlines() if line.strip() and not line.strip().startswith('#')]

# Make sure we are running python3.5+
if 10 * sys.version_info[0]  + sys.version_info[1] < 35:
    sys.exit("Sorry, only Python 3.5+ is supported.")


def readme():
    with open('README.rst') as f:
        return f.read()

setup(
      name             =   'pfmongo',
      version          =   get_version('pfmongo/__main__.py'),
      description      =   'A mongodb (portable) "pf" client',
      long_description =   readme(),
      author           =   'FNNDSC',
      author_email     =   'dev@babymri.org',
      url              =   'https://github.com/FNNDSC/persair',
      packages         =   ['pfmongo',
                            'pfmongo/commands',
                            'pfmongo/commands/fop',
                            'pfmongo/commands/dbop',
                            'pfmongo/commands/clop',
                            'pfmongo/commands/stateop',
                            'pfmongo/commands/slib',
                            'pfmongo/db',
                            'pfmongo/config',
                            'pfmongo/models'],
      install_requires  = requirements,
      data_files        =   [
          ('', ['requirements.txt']),
        ],
      entry_points={
          'console_scripts': [
              'pfmongo = pfmongo.__main__:main'
          ]
      },
      license          =   'MIT',
      zip_safe         =   False
)



from pfmongo import pfmongo
from pfmongo.__main__ import main
import os, pudb

from pfmongo.commands.dbop import connect as database, deleteDB
from pfmongo.commands.clop import connect as collection
from pfmongo.commands.docop import add, get
from pfmongo.models import responseModel
from pfmongo.config import settings
from pfmongo import driver
import json

os.environ["XDG_CONFIG_HOME"] = "/tmp"


def DB_connect(DB: str = "testDB") -> int:
    return database.connectTo_asInt(
        database.options_add(DB, pfmongo.options_initialize())
    )


def collection_connect(col: str = "testCollection") -> int:
    return collection.connectTo_asInt(
        collection.options_add(col, pfmongo.options_initialize())
    )


def DB_delete(DB: str = "testDB") -> int:
    return deleteDB.DBdel_asInt(deleteDB.options_add(DB, pfmongo.options_initialize()))


def test_document_add_asInt() -> None:
    DB_delete()
    DB_connect()
    collection_connect()
    retlld: int = add.documentAdd_asInt(
        add.options_add("examples/lld.json", "lld.json", pfmongo.options_initialize())
    )
    retneuro1: int = add.documentAdd_asInt(
        add.options_add(
            "examples/neuro1.json", "neuro1.json", pfmongo.options_initialize()
        )
    )
    retneuro2: int = add.documentAdd_asInt(
        add.options_add(
            "examples/neuro2.json", "neuro2.json", pfmongo.options_initialize()
        )
    )
    retultrasound: int = add.documentAdd_asInt(
        add.options_add(
            "examples/ultrasound.json", "ultrasound.json", pfmongo.options_initialize()
        )
    )
    assert retlld == 0
    assert retneuro1 == 0
    assert retneuro2 == 0
    assert retultrasound == 0
    DB_delete()


def test_duplicate_add_asInt() -> None:
    DB_delete()
    DB_connect()
    collection_connect()
    retlld: int = add.documentAdd_asInt(
        add.options_add("examples/lld.json", "lld.json", pfmongo.options_initialize())
    )
    assert retlld == 0
    retlld = add.documentAdd_asInt(
        add.options_add("examples/lld.json", "lld.json", pfmongo.options_initialize())
    )
    assert retlld == 103
    DB_delete()


def test_duplicateID_add_asModel() -> None:
    DB_delete()
    DB_connect()
    collection_connect()
    retlld: responseModel.mongodbResponse = add.documentAdd_asModel(
        add.options_add(
            "examples/lld.json",
            "lld.json",
            pfmongo.options_initialize([{"noHashing": True}]),
        )
    )
    retlld = add.documentAdd_asModel(
        add.options_add(
            "examples/lld.json",
            "lld.json",
            pfmongo.options_initialize([{"noHashing": True}]),
        )
    )
    assert "Could not add" in retlld.message
    assert not retlld.response["status"]
    assert "E11000 duplicate key" in retlld.response["connect"].resp["error"]
    DB_delete()


def test_duplicateHash_add_asModel() -> None:
    DB_delete()
    DB_connect()
    collection_connect()
    retlld: responseModel.mongodbResponse = add.documentAdd_asModel(
        add.options_add("examples/lld.json", "lld2.json", pfmongo.options_initialize())
    )
    retlld = add.documentAdd_asModel(
        add.options_add("examples/lld.json", "lld2.json", pfmongo.options_initialize())
    )
    assert "Could not add" in retlld.message
    assert not retlld.response["status"]
    assert "Duplicate document hash found." in retlld.response["connect"].resp["error"]
    DB_delete()


def test_document_get_asInt() -> None:
    DB_delete()
    DB_connect()
    collection_connect()
    load: int = add.documentAdd_asInt(
        add.options_add("examples/lld.json", "lld.json", pfmongo.options_initialize())
    )
    assert load == 0
    read: int = get.documentGet_asInt(
        get.options_add("lld.json", pfmongo.options_initialize())
    )
    assert read == 0
    DB_delete()


def test_document_get_asModel() -> None:
    DB_delete()
    DB_connect()
    collection_connect()
    load: int = add.documentAdd_asInt(
        add.options_add("examples/lld.json", "lld.json", pfmongo.options_initialize())
    )
    assert load == 0
    read: responseModel.mongodbResponse = get.documentGet_asModel(
        get.options_add("lld.json", pfmongo.options_initialize())
    )
    d_read: dict[str, str] = json.loads(read.message)
    assert d_read["_id"] == "lld.json"
    DB_delete()


def test_deleteTestDB() -> None:
    ret: int = deleteDB.DBdel_asInt(
        deleteDB.options_add("testDB", pfmongo.options_initialize())
    )
    assert ret == 0


if __name__ == "__main__":
    print("Test document operations")
    pudb.set_trace()
    test_document_get_asModel()
